package diffcad;

import futils.Print;

import java.awt.*;

class Laser extends Shape {
    double W = 10;
    double H = 20;

    void print() {
        Print.className(this);
        print("Rotation= " + rotation.getValue() + "\t");
        pc.println();
        // output to console, not file
        printNG();
    }

    // print the distance between the intersection of the laser
    // ray and the origin.
    void printNG() {
        System.out.println("Laser NG =" + NG());
    }

    double NG() {
        double alpha = compute_alpha_radians();
        double x2 = pc.x;
        double y2 = pc.y;
        return -x2 - y2 * Math.tan(alpha);
    }

    double compute_alpha_radians() {
        return rotation.getValue() * Math.PI / 180;
    }

    void update() {

        pc.x = p1.x + W / 2.0;
        pc.y = p1.y;
        p2.x = p1.x + W;
        p2.y = p1.y;

        p3.x = p2.x;
        p3.y = p1.y - H;

        p4.x = p1.x;
        p4.y = p3.y;

        pb.x = pc.x;
        pb.y = p4.y;

        pointRotation(p1);
        pointRotation(p2);
        pointRotation(p3);
        pointRotation(p4);
        pointRotation(pb);
    }

    void change_config(double x, double y) {
        p1.x = x;
        p1.y = y;
        update();
    }

    void change_config(point t) {
        change_config(p1.x + t.x, p1.y + t.y);
    }

    void draw(Graphics g) {
        pushGraphicsState(g);
        // p1 -pc- p2
        //  |      |
        // p4      p3
        //   - pb-
        // Mouse click comes in x1 and y1, upper left corner

        line(g, p1, p2);
        line(g, p2, p3);
        line(g, p3, p4);
        line(g, p4, p1);
        line(g, pc, pb);
        string(g, "PL", pc);
        popGraphicsState(g);
    }
}

