package grapher;

import java.io.InputStream;
import java.io.StreamTokenizer;
import java.util.Hashtable;


/*************************************************************************
**
**    Class  ScanWord
**                                              Version 1.0   January  1995
**
**************************************************************************
**    Copyright (C) 1996 Leigh Brookshaw
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program; if not, write to the Free Software
*    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**************************************************************************
**
** This class extends the StreamTokenizer class. It allows words
** to be defined and recognised and returned as a token. The TT_NUMBER
** token of the StreamTokenizer is also modified so that scientific notation
** is recognised.
**
 *************************************************************************/

public class ScanWord extends StreamTokenizer {
    /********************
    ** Constants
     ********************/
    public final static int UNKNOWN = -256;
    public final static int ERROR = -257;
    public final static int STRING = -258;
    /********************
    ** Public Variables
     ********************/

    /********************
    ** Private Variables
     ********************/

    private InputStream is = null;
/*
**   The hash table containing the keyword/value pairs.
*/
    private Hashtable kwords = new Hashtable();
/*
**   The character that delimeters a string
*/
    private char StringChar = '"';

    /*****************
    **
    ** Constructors
    **
     *****************/

    public ScanWord(InputStream in) {
        super(in);
        this.is = in;
        resetWordSyntax();
    }

    /******************
    **
    **  Public Methods
    **
     *******************/

/*
**   Add a keyword/value pair to the hash table
*/
    public void addKeyWord(String s, int i) {

        if (s == null) return;

        kwords.put(s.toLowerCase(), new Integer(i));

    }
/*
**   return the value given the keyword
*/
    public int getKeyValue(String s) {

        if (s == null) return UNKNOWN;

        Integer i = (Integer) kwords.get(s.toLowerCase());

        if (i == null)
            return UNKNOWN;
        else
            return i.intValue();
    }
/*
**   Clear the hash table containing the keyword/value pairs
*/
    public void resetKeyWords() {
        kwords.clear();
    }
/*
**  Process the input stream and return the next token found.
*/
    public int nextWord() {
        int word;

        try {
            ttype = nextToken();
        } catch (Exception e) {
            return ERROR;
        }

        if (ttype == StringChar) {
            return STRING;
        } else if (ttype == TT_WORD) {
            word = getKeyValue(super.sval);
            if (word == UNKNOWN) word = getNumber(super.sval);
            if (word == UNKNOWN) return ttype;
            ttype = word;
            return word;
        } else {
            return ttype;
        }
    }

/*
** Set the character that delimeters a string.
*/
    public void setStringChar(char c) {
        StringChar = c;
        quoteChar(StringChar);
    }



    /********************
    **
    ** Protected Methods
    **
     ********************/

/*
**   Check to see if the returned word is a number
**   The native StreamTokenizer number method does not work for
**   scientific notation so we do it this way
**
*/
    protected int getNumber(String s) {

        try {
            super.nval = Double.valueOf(s).doubleValue();
            return TT_NUMBER;
        } catch (Exception e) {
            return UNKNOWN;
        }

    }

/*
**   Reset the syntax of the parent class
*/
    protected void resetWordSyntax() {
        resetSyntax();
        wordChars('a', 'z');
        wordChars('A', 'Z');
        wordChars('0', '9');
        wordChars('.', '.');
        wordChars('_', '_');
        wordChars('{', '}');
        wordChars(128 + 32, 255);

        quoteChar(StringChar);

        whitespaceChars(0, ' ');

        commentChar('#');

        lowerCaseMode(false);

        eolIsSignificant(true);
    }
/*
**  Close the input stream
*/

    protected void closeStream() {
        try {
            is.close();
        } catch (Exception e) {
        }

        is = null;
    }

}
